<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:java="http://xml.apache.org/xslt/java"
	xmlns:util="xalan://org.greenstone.gsdl3.util.XSLTUtil"
	xmlns:gslib="http://www.greenstone.org/skinning"
	xmlns:gsf="http://www.greenstone.org/greenstone3/schema/ConfigFormat"
	extension-element-prefixes="java util"
	exclude-result-prefixes="java util gsf">

  <!--
      Key goal: generate a IIIF v2 compliant manifest file in JSON format based on the document
  -->
  
  <!-- Similar in spirit to document.xsl (catering for 'simple' and complex document structures such as 'pages'),
       but considerably tailored down to produce the raw IIIF-manifest JSON format:
         (i)  focus on just the (potientailly hierachically nested) pages
         (ii) so a bit like ed=1, but implied rather than needing to be set
  -->

  <xsl:include href="layouts/header-core.xsl"/>

  <xsl:include href="pages/document-shared.xsl" />
  <xsl:include href="pages/document-core.xsl" />

  <xsl:output method="html" version="1.0" encoding="utf-8"/>
  
  <!--
  <xsl:output method="text" version="1.0" encoding="utf-8" omit-xml-declaration="yes"/>
  <xsl:strip-space elements="*"/>
  -->
    
  <xsl:variable name="iiifPresentationRootIdentifier">http-greenstone://<xsl:value-of select="$site_name"/>/<xsl:value-of select="$collNameChecked"/>/<xsl:value-of select="$docID"/></xsl:variable>

  <!-- **** should be able to get rid of this, however at the moment it seems to be used in some supporting file !!!! -->
  
  <xsl:variable name="bookswitch">mirador</xsl:variable>
    
  <xsl:template match="/">
    <html>
      <body>
	<xsl:apply-templates select="/page/pageResponse/document" />
      </body>
    </html>
    
  </xsl:template>


  <xsl:template name="iiif-other-content">
    <xsl:param name="optSection"/>
    
    <xsl:variable name="baseURL">https:<xsl:value-of select="/page/pageRequest/@baseURL"/></xsl:variable> <!-- **** had to hardwire https: -->    
    <!-- **** test for a gsf-option before include !!!! -->    
  	      "otherContent": [{
		    "@id": "<xsl:value-of select="$baseURL"/><xsl:value-of select="$library_name"/>/sites/<xsl:value-of select="$site_name"/>/collect/<xsl:value-of select="$collNameChecked"/>/index/assoc/<gsf:metadata name="assocfilepath" select="root"/>/openannotation-list<xsl:value-of select="$optSection"/>.json",
                    "@type": "sc:AnnotationList"
              }],
  </xsl:template>
  

  <!-- **** improve name !!!! -->
  
  <xsl:template name="iiif-links">
    <xsl:param name="identifier"/>
    <xsl:param name="docType"/>
    
    <xsl:variable name="iiifImageRootIdentifier">
      <xsl:text>/gs-cantaloupe/iiif/2/</xsl:text><xsl:value-of select="$site_name"/>:<xsl:value-of select="$collNameChecked"/>:<xsl:value-of select="$identifier"/>
    </xsl:variable>
    
    <!-- page/pageRequest[@baseURL]="//intermuse.sowemustthink.space/greenstone3/ -->
    <!-- page/pageRequest[@fullURL]="//intermuse.sowemustthink.space/greenstone3/library?a=d.... -->

    <!-- ***** need to work out domain name to use from raw XML message, say processing something from the above -->
    
    <xsl:variable name="document-location-origin">https://intermuse.sowemustthink.space</xsl:variable>

    <xsl:variable name="section"><xsl:value-of select="position()" /></xsl:variable>
    <xsl:variable name="optSection">
      <xsl:choose>
	<xsl:when test="$docType = 'simple'"></xsl:when><!-- i.e. empty -->
	<xsl:otherwise><xsl:value-of select="position()" /></xsl:otherwise>
      </xsl:choose>
    </xsl:variable>


    <xsl:variable name="chosenTitle">
      <xsl:choose>
	<xsl:when test="$docType = 'simple'">
	  <xsl:call-template name="choose-title"/>
	</xsl:when>
	<xsl:otherwise>page <xsl:value-of select="$section"/></xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <!-- The following is useful for debugging, nested in 'pre' tags -->
    <!--
      docType   = <xsl:value-of select="$docType"/>
      docID     = <xsl:value-of select="$docID"/>
      position  = <xsl:value-of select="position()" />

      identifer = <xsl:value-of select="$identifier"/>
      section    = <xsl:value-of select="$section"/>
      optSection = <xsl:value-of select="$optSection"/>

      iiifPresRootIdentifier  = <xsl:value-of select="$iiifPresentationRootIdentifier" />
      iiifImageRootIdentifier = <xsl:value-of select="$iiifImageRootIdentifier" />
    -->
    <!-- an array entry in 'canvases': { @content, @id, @type, label, width, height, images[{}], thumbnail {@id,@type} } -->
    <pre>
      <!-- add a comma first, if it is not the first canvas element in the array -->
      <xsl:if test="$section &gt; 1">
	  ,
      </xsl:if>
	    {
	      "@context": "http://iiif.io/api/presentation/2/context.json",
	      "@id":      "<xsl:value-of select="$iiifPresentationRootIdentifier" />/canvas/<xsl:value-of select="$section"/>",
	      "@type":    "sc:Canvas",
	      "label":    "<xsl:value-of select="$chosenTitle"/>",
	      <xsl:call-template name="iiif-other-content">
		<xsl:with-param name="optSection"><xsl:value-of select="$optSection"/></xsl:with-param>
	      </xsl:call-template>	    
              &quot;width&quot;:  <gsf:metadata name="ImageWidth"/>,
              &quot;height&quot;: <gsf:metadata name="ImageHeight"/>,
	      
	      "images": [{
                "image": {
	          "@context":   "http://iiif.io/api/image/2/context.json",
	          "@type":      "oa:Annotation",
		  "motivation": "sc:painting",
		  "on":         "<xsl:value-of select="$iiifPresentationRootIdentifier" />/annotation/<xsl:value-of select="$section"/>",

		  "resource": {
		    "@id":     "<xsl:value-of select="$document-location-origin" /><xsl:value-of select="$iiifImageRootIdentifier" />/full/full/0/default.jpg",
		    "@type":   "dctypes:Image",
		    "format":  "image/jpeg",
		    "width":   <gsf:metadata name="ImageWidth"/>,
		    "hieght":  <gsf:metadata name="ImageHeight"/>,
		    "service": {
		      "@context": "http://iiif.io/api/image/2/context.json",
		      "@id":      "<xsl:value-of select="$document-location-origin" /><xsl:value-of select="$iiifImageRootIdentifier" />",
		      "profile":  "https://iiif.io/api/image/2/profiles/level2.json"
		    }
		  }
		}
	      }],
	      "thumbnail": {
	        "@id":   "<xsl:value-of select="$document-location-origin" /><xsl:value-of select="$iiifImageRootIdentifier" />/full/,150/0/default.jpg",
		"@type": "dctypes:Image"
	      }
            }      
    </pre>
    <!-- Another field to consider in the above JSON is: -->
    <!--   "format": mime_type, // ???? or is it OK to always have this as image/jpeg and rely on IIIF Image Server coverting as/when needed??? -->
  </xsl:template>


  <xsl:template name="wrappedSectionPost">

    <!-- add in IIIF Server link for section image -->
    <xsl:variable name="nodeID" select="@nodeID"/>
    <xsl:call-template name="iiif-links">
      <xsl:with-param name="identifier" select="$nodeID" />
      <xsl:with-param name="docType" select="@docType" />
    </xsl:call-template>
  </xsl:template>

  <xsl:template name="sectionImage">
    <!-- don't need this template to generate anything in the IIIF-Manifest JSON case -->
  </xsl:template>

  <xsl:template name="sectionContent">
    <xsl:call-template name="wrappedSectionPost"/>
  </xsl:template>

  
  <xsl:template name="documentPre">
    <xsl:variable name="httpCollection">library/sites/<xsl:value-of select="$site_name"/>/collect/<xsl:value-of select="/page/pageResponse/collection/@name"/></xsl:variable>

    <xsl:variable name="chosenTitle">
      <xsl:choose>
	<xsl:when test="@docType='simple'">
	  <xsl:call-template name="choose-title"/>
	</xsl:when>
	<xsl:otherwise>
	  <xsl:value-of disable-output-escaping="yes" select="/page/pageResponse/document/documentNode/metadataList/metadata[@name='Title']" />
	</xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <!-- start of iiif_manifest: -->
    <pre>
    {
        "@context": "http://iiif.io/api/presentation/2/context.json",
	"@type": "sc:Manifest",
	"@id": "<xsl:value-of select="$iiifPresentationRootIdentifier"/>/manifest",
	"label": "<xsl:value-of select="$chosenTitle"/>",
	"attribution": "The Internet of Musical Events (InterMusE): Digital Scholarship, Community, and the Archiving of Performance",
	"license": "https://**** To Be Determined ****",
	"logo": "https://intermuse.datatodata.org/wp-content/uploads/sites/4/2021/03/cropped-cropped-cropped-1.png",
        "metadata": [
	  { "label": "creator", "value": ["aaa1","aaa2"] },
	  { "label": "title", "value": ["chosen-title"] }
	],
	<!-- trival IIIF sequences => has one IIIF canvas in it -->
	"sequences": [{
	  "@context": "http://iiif.io/api/image/2/context.json",
          "@id": "<xsl:value-of select="$iiifPresentationRootIdentifier"/>/sequence/normal",
          "@type": "sc:Sequence",
	  "canvases": [
    </pre>

  </xsl:template>
     
  
  <xsl:template name="documentContentSimpleIIIFImage">
    <xsl:call-template name="iiif-links">
      <xsl:with-param name="identifier" select="$docID" />
      <xsl:with-param name="docType" select="@docType" />
    </xsl:call-template>
  </xsl:template>
  
  <xsl:template name="documentContent">
    <!-- The way document.xsl templates are structured, only get to here is if @docType = 'simple' -->
    <!-- A 'paged' (or 'hierarical') document goes throgh 'wrappedContent' -->
    <xsl:call-template name="documentPre"/>
    <xsl:call-template name="documentContentSimpleIIIFImage"/>
    <xsl:call-template name="documentPost"/>
  </xsl:template>

  <xsl:template name="documentPost">
    <!-- end of iiif_manifest: -->
    <!-- close of canvases, then close higher level blocks to complete the JSON file -->
    <pre>
	  ]
	}]	  
    }    	
    </pre>
    
  </xsl:template>
  

  <xsl:template name="wrappedDocumentTopLevel">

    <!-- <xsl:call-template name="documentPre"/> -->
    <xsl:call-template name="wrappedDocument"/> <!-- without ed=1, this template misses the binds in of documentPre and documentPost -->
    <!-- <xsl:call-template name="documentPost"/> -->
    <!--
    <xsl:call-template name="documentPostMirador3OLD"/>
    -->

  </xsl:template>


	<xsl:template name="wrapDocumentNodes">

	  <xsl:choose>
	    <xsl:when test="../../document">
	      <xsl:call-template name="topLevelSectionContent"/>
	    </xsl:when>
	    <xsl:otherwise>
	      <xsl:call-template name="sectionContent"/>
	    </xsl:otherwise>
	  </xsl:choose>
	  
	  <xsl:if test="documentNode">
	    <xsl:for-each select="documentNode">
	      <xsl:call-template name="wrapDocumentNodes"/>
	    </xsl:for-each>
	  </xsl:if>
	</xsl:template>



	<xsl:template name="wrappedSectionImage">
	  <!-- don't need this template to generate anything in the IIIF-Manifest JSON case -->
	</xsl:template>
	
	<!-- The default template for displaying the document node text -->
	<!-- equivalent to gsf:text -->
	<xsl:template name="documentNodeText">
	  <xsl:param name="force">0</xsl:param>
		<!-- Hides the "This document has no text." message -->
		<xsl:variable name="noText"><gsf:metadata name="NoText"/></xsl:variable>
		<xsl:choose>
		<xsl:when test="$force = '1' or not($noText = '1')">

			<!-- Section text -->
			<xsl:for-each select="nodeContent">
			  <xsl:call-template name="displayMarkedUpTextAndAnnotations"/>
			</xsl:for-each>
		</xsl:when>
		<xsl:when test="$noText = '1' and not(metadataList/metadata[@name='ImageType'])">
			<gsf:link type="source"><gsf:metadata name="Source"/></gsf:link>
		</xsl:when>
		</xsl:choose>
		<xsl:text> </xsl:text>
	</xsl:template>





	<xsl:template match="/page/pageResponse/document">
		<!-- display the document -->
		<xsl:choose>
			<xsl:when test="@docType='simple'">
				<xsl:call-template name="documentHeading"/><br/>
				<xsl:call-template name="documentContentTopLevel"/>
			</xsl:when>	
			<xsl:otherwise> <!-- display the standard greenstone document -->
				<xsl:call-template name="wrappedDocumentTopLevel"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>


	<!-- **** SIMPLIFY !!!! -->

	<xsl:template name="wrappedDocument">
		<xsl:choose>
			<!-- NOTE: alb = ajax load bypass -->
			<!-- 
			     If we have asked for expanded document, then do this.
			     OR If the docType is hierarchy and we want to bypass the ajax load then do this 
			     OR If the docType is hierarchy and we have asked for the top level document then do this 
			-->
			<xsl:when test="/page/pageRequest/paramList/param[@name = 'ed']/@value = '1' or (/page/pageResponse/document/@docType = 'hierarchy' and (/page/pageRequest/paramList/param[@name = 'alb']/@value = '1' or (string-length(/page/pageRequest/paramList/param[@name = 'd']/@value) > 0 and not(util:contains(/page/pageResponse/document/@selectedNode, '.')))))">
				<div id="gs-document">
					<xsl:call-template name="documentPre"/>
					<div id="gs-document-text" class="documenttext" collection="{/page/pageResponse/collection/@name}"><!-- *** -->
					  <xsl:for-each select="documentNode">
							<xsl:call-template name="wrapDocumentNodes"/>
						</xsl:for-each>
					</div>
					<xsl:call-template name="documentPost"/>					
				</div>
			</xsl:when>
			<xsl:when test="/page/pageResponse/document/@docType = 'paged' or /page/pageResponse/document/@docType = 'pagedhierarchy'">
			  <div id="gs-document">
					<div id="tocLoadingImage" style="text-align:center;">
						<img src="{util:getInterfaceText($interface_name, /page/@lang, 'loading_image')}"/><xsl:value-of select="util:getInterfaceText($interface_name, /page/@lang, 'doc.loading')"/><xsl:text>...</xsl:text>
					</div>
				</div>
				<script type="text/javascript">
					<xsl:text disable-output-escaping="yes">
						$(window).on("load", function()
						{
							var sectionID = gs.cgiParams.d;
							var callbackFunction = null;
							if(sectionID.indexOf(".") != -1)
							{
								callbackFunction = function()
								{
									focusSection(sectionID);
								};
							}
							else {
							   callbackFunction = function()
								{
								
								expandAndExecute(sectionID+".1", null, null, null);
								};
							}
							var docID = sectionID.replace(/([^.]*)\..*/, "$1");
							var url = gs.xsltParams.library_name + "?a=d&amp;c=" + gs.cgiParams.c + "&amp;excerptid=gs-document&amp;dt=hierarchy&amp;d=" + docID;
							if(gs.cgiParams.p_s) { 
							url += "&amp;p.s="+gs.cgiParams.p_s;
							}
							loadTopLevelPage(callbackFunction, url);
						});
					</xsl:text>
				</script>
			</xsl:when>
			<xsl:otherwise>
				<div id="gs-document">							
					<div id="tocLoadingImage" style="text-align:center;">
						<img src="{util:getInterfaceText($interface_name, /page/@lang, 'loading_image')}"/><xsl:value-of select="util:getInterfaceText($interface_name, /page/@lang, 'doc.loading')"/><xsl:text>...</xsl:text>
					</div>
				</div>
				<script type="text/javascript">
					<xsl:text disable-output-escaping="yes">
						$(window).on("load", function()
						{
							loadTopLevelPage(function()
							{
								//Don't focus the section until the table of contents is loaded
								var tocCheck = function()
								{
									if(gs.variables.tocLoaded)
									{
										focusSection("</xsl:text><xsl:value-of select="/page/pageResponse/document/@selectedNode"/><xsl:text disable-output-escaping="yes">");
									}
									else
									{
										setTimeout(tocCheck, 500);
									}
								}
								tocCheck();
							});
						});
					</xsl:text>
				</script>
			</xsl:otherwise>
		</xsl:choose>
		
		<div class="clear"><xsl:text> </xsl:text></div>
	</xsl:template>



	
</xsl:stylesheet>
